@@ -22,8 +22,10 @@
 #include "ngdata.h"
 #include "ng.h"
 #include "util.h"
+#include "util2.h"
 #include "trn.h"
 #include "artio.h"
+#include "nntp.h"
 #include "final.h"
 #include "decode.h"
 #include "charsubst.h"
@@ -37,7 +39,8 @@ bool cut_line();
 void
 respond_init()
 {
-    export("NEWSACTIVE", filexp(ACTIVE));
+    if (actfp)
+	export("NEWSACTIVE", filexp(ACTIVE));
 }
 
 int
@@ -124,7 +127,7 @@ Saving null articles is not very productive!  :-)\n\
 	}
 	custom_extract = (cmdstr != 0);
 
-	fseek(artfp,savefrom,0);
+	seekart((long)savefrom);
 	if (*s != '/') {		/* relative path? */
 	    c = (s==buf ? altbuf : buf);
 	    interp(c, (sizeof buf), getval("SAVEDIR",SAVEDIR));
@@ -146,6 +149,9 @@ Saving null articles is not very productive!  :-)\n\
 	    printf("Continuing %s:%s\n", decode_fname,
 		cmd != '\0' && strNE(extractdest,s) ?
 		 " (Ignoring conflicting directory)" : nullstr ) FLUSH;
+#ifdef USE_NNTP
+	    nntp_finishbody(FB_SILENT);
+#endif
 	    if (decode_type == UUDECODE)
 		uudecode(artfp);
 	    else
@@ -157,6 +163,9 @@ Saving null articles is not very productive!  :-)\n\
 	    s = extractdest = savestr(s); /* make it handy for %E */
 	    if (makedir(s, MD_DIR)) {	/* ensure directory exists */
 		int_count++;
+#ifdef USE_NNTP
+		nntp_finishbody(FB_SILENT);
+#endif
 		return SAVE_DONE;
 	    }
 	    if (chdir(s)) {
@@ -164,7 +173,7 @@ Saving null articles is not very productive!  :-)\n\
 		sig_catcher(0);
 	    }
 	    s = getwd(buf);		/* simplify path for output */
-	    while(fgets(art_buf,LBUFLEN,artfp) != Nullch) {
+	    while(readart(art_buf,LBUFLEN) != Nullch) {
 		if (*art_buf <= ' ')
 		    continue;	/* Ignore empty or initially-whitespace lines */
 #ifdef MIMESTORE
@@ -178,6 +187,9 @@ Saving null articles is not very productive!  :-)\n\
 			printf("Extracting MIME article into %s using %s\n",
 			       s, extractprog) FLUSH;
 		    cnt = 0;
+#ifdef USE_NNTP
+		    nntp_finishbody(FB_SILENT);
+#endif
 		    interp(cmd_buf, sizeof cmd_buf,
 			   getval("EXMIMESAVER",EXMIMESAVER));
 		    termlib_reset();
@@ -203,15 +215,15 @@ Saving null articles is not very productive!  :-)\n\
 		 || strnEQ(art_buf, "sed ", 4)
 		 || strnEQ(art_buf, "cat ", 4)
 		 || strnEQ(art_buf, "echo ", 5)) {
-		    fseek(artfp,-(long)strlen(art_buf)-NL_SIZE+1,1);
-		    savefrom = ftell(artfp);
+		    savefrom = tellart()-strlen(art_buf)-NL_SIZE+1;
+		    seekart((long)savefrom);
 		    if (custom_extract) {
 			printf("Extracting shar into %s using %s:\n",
 				s, extractprog) FLUSH;
 			goto extract_it;
 		    }
 		    /* Check for special-case of shar'ed-uuencoded file */
-		    while(fgets(art_buf,LBUFLEN,artfp) != Nullch) {
+		    while(readart(art_buf,LBUFLEN) != Nullch) {
 			if (*art_buf == '#' || *art_buf == ':'
 			 || strnEQ(art_buf, "echo ", 5)
 			 || strnEQ(art_buf, "sed ", 4))
@@ -228,6 +240,9 @@ Saving null articles is not very productive!  :-)\n\
 		    extractprog = savestr(filexp(getval("UNSHAR",UNSHAR)));
 		  extract_it:
 		    cnt = 0;
+#ifdef USE_NNTP
+		    nntp_finishbody(FB_SILENT);
+#endif
 		    interp(cmd_buf,(sizeof cmd_buf),getval("EXSAVER",EXSAVER));
 		    termlib_reset();
 		    resetty();		/* restore tty state */
@@ -241,9 +256,9 @@ Saving null articles is not very productive!  :-)\n\
 		if (!custom_extract
 		 && (strEQ(art_buf,"$\n")
 		  || strEQ(art_buf,"$ f\n"))) {
-		    savefrom = ftell(artfp)-strlen(art_buf)-NL_SIZE+1;
+		    savefrom = tellart()-strlen(art_buf)-NL_SIZE+1;
 		    if (found_cut
-		     || (fgets(art_buf,LBUFLEN,artfp) != Nullch
+		     || (readart(art_buf,LBUFLEN) != Nullch
 		      && (strnEQ(art_buf, "ship ", 5)
 		       || strnEQ(art_buf, "cont ", 5)))) {
 			decode_type = UNSHIP;
@@ -255,15 +270,18 @@ Saving null articles is not very productive!  :-)\n\
 		 && (strEQ(art_buf,"table\n")
 		  || strnEQ(art_buf,"begin ", 6))) {
 		    decode_type = UUDECODE;
-		    savefrom = ftell(artfp)-strlen(art_buf)-NL_SIZE+1;
+		    savefrom = tellart()-strlen(art_buf)-NL_SIZE+1;
 		 decode_it:
 		    printf("Extracting %s file into %s:\n",
 			decode_type == UNSHIP? "shipped":"uuencoded", s) FLUSH;
 		    if (extractprog)
 			free(extractprog);
 		    extractprog = savestr("-");
-		    fseek(artfp, savefrom, 0);
+		    seekart((long)savefrom);
 		    cnt = 0;
+#ifdef USE_NNTP
+		    nntp_finishbody(FB_SILENT);
+#endif
 		    if (decode_type == UUDECODE) {
 			uud_start();
 			uudecode(artfp);
@@ -273,7 +291,7 @@ Saving null articles is not very productive!  :-)\n\
 		}
 		else {
 		    if (cut_line(art_buf)) {
-			savefrom = ftell(artfp);
+			savefrom = tellart();
 			found_cut = TRUE;
 		    }
 		    else if (found_cut || ++cnt == 300) {
@@ -298,6 +316,9 @@ Saving null articles is not very productive!  :-)\n\
 	if (savedest)
 	    free(savedest);
 	savedest = savestr(altbuf);
+#ifdef USE_NNTP
+	nntp_finishbody(FB_SILENT);
+#endif
 	interp(cmd_buf, (sizeof cmd_buf), getval("PIPESAVER",PIPESAVER));
 				/* then set up for command */
 	termlib_reset();
@@ -439,6 +460,9 @@ q to abort.\n\
 	    getval("MBOXSAVER",MBOXSAVER) :
 	    getval("NORMSAVER",NORMSAVER) ), sizeof cmd_buf);
 				/* format the command */
+#ifdef USE_NNTP
+	nntp_finishbody(FB_SILENT);
+#endif
 	termlib_reset();
 	resetty();		/* make terminal behave */
 	if (doshell(use_pref?Nullch:SH,cmd_buf)) {
@@ -465,6 +489,9 @@ s_bomb:
 	printf(nocd,ngdir) FLUSH;
 	sig_catcher(0);
     }
+#ifdef USE_NNTP
+    nntp_finishbody(FB_SILENT);
+#endif
     return SAVE_DONE;
 }
 
@@ -590,8 +617,8 @@ Superceding null articles is your idea of fun?  :-)\n\
 	fputs(buf,tmpfp);
 	if (incl_body && artfp != Nullfp) {
 	    parseheader(art);
-	    fseek(artfp,(long)htype[PAST_HEADER].ht_minpos,0);
-	    while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+	    seekart((long)htype[PAST_HEADER].ht_minpos);
+	    while (readart(buf,LBUFLEN) != Nullch) {
 		fputs(buf,tmpfp);
 	    }
 	}
@@ -639,8 +666,8 @@ reply()
 	interp(buf, (sizeof buf), getval("YOUSAID",YOUSAID));
 	fprintf(tmpfp,"%s\n",buf);
 	parseheader(art);
-	fseek(artfp,(long)htype[PAST_HEADER].ht_minpos,0);
-	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+	seekart((long)htype[PAST_HEADER].ht_minpos);
+	while (readart(buf,LBUFLEN) != Nullch) {
 #ifdef CHARSUBST
 	    strcharsubst(buf,hbuf);
 	    fprintf(tmpfp,"%s%s",indstr,hbuf);
@@ -689,8 +716,8 @@ forward()
 	if (*buf)
 	    fprintf(tmpfp,"%s\n",buf);
 	parseheader(art);
-	fseek(artfp,0L,0);
-	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+	seekart(0L);
+	while (readart(buf,LBUFLEN) != Nullch) {
 #ifdef CHARSUBST
 	    strcharsubst(buf,hbuf);
 	    fprintf(tmpfp,"%s",hbuf);
@@ -745,8 +772,8 @@ trim the quoted article down as much as possible.)\n\
 	interp(buf, (sizeof buf), getval("ATTRIBUTION",ATTRIBUTION));
 	fprintf(tmpfp,"%s\n",buf);
 	parseheader(art);
-	fseek(artfp,(long)htype[PAST_HEADER].ht_minpos,0);
-	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+	seekart((long)htype[PAST_HEADER].ht_minpos);
+	while (readart(buf,LBUFLEN) != Nullch) {
 #ifdef CHARSUBST
 	    strcharsubst(buf,hbuf);
 	    fprintf(tmpfp,"%s%s",indstr,hbuf);
